//+------------------------------------------------------------------+
//|                                                FractalLevels.mq4 |
//|                                        Copyright  2008, lotos4u |
//|                                                lotos4u@gmail.com |
//+------------------------------------------------------------------+
#property copyright "Copyright  2008, lotos4u"
#property link      "lotos4u@gmail.com"

#property indicator_chart_window
#property indicator_buffers 4

#property indicator_width1 4
#property indicator_width2 4
#property indicator_width3 2
#property indicator_width4 2

#property indicator_color1 Red
#property indicator_color2 Blue
#property indicator_color3 Gold
#property indicator_color4 Gold

//---- input parameters
extern string TimeFrame       = "Current time frame";
extern int  FractalBars     = 3;
extern bool alertsOn        = true;
extern bool alertsOnCurrent = true;
extern bool alertsMessage   = true;
extern bool alertsSound     = false;
extern bool alertsEmail     = false;

//---- buffers
double ExtMapBuffer1[];
double ExtMapBuffer2[];
double ArrowUpBuffer3[];
double ArrowDownBuffer4[];

string indicatorFileName;
bool   returnBars;
int    timeFrame;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
   SetIndexBuffer(0, ExtMapBuffer1);
   SetIndexLabel(0, " ");
   
   SetIndexBuffer(1, ExtMapBuffer2);
   SetIndexLabel(1, " ");

   SetIndexStyle(2, DRAW_ARROW);
   SetIndexArrow(2, 119);
   SetIndexBuffer(2, ArrowUpBuffer3);
   SetIndexLabel(2, " ");
   
   SetIndexStyle(3, DRAW_ARROW);
   SetIndexArrow(3, 119);
   SetIndexBuffer(3, ArrowDownBuffer4);
   SetIndexLabel(3, " ");
         timeFrame         = stringToTimeFrame(TimeFrame);
         indicatorFileName = WindowExpertName();
         returnBars        = TimeFrame=="returnBars";
   return(0);
}


//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit(){return(0);}



//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
           int limit=MathMin(MathMax(Bars-counted_bars,FractalBars),Bars-1);
           if (returnBars) { ExtMapBuffer1[0] = MathMin(limit+1,Bars-1); return(0); }
           if (timeFrame!=Period())
           {
               limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
               for(int i = limit; i > 0; i--)
               {
                  int y = iBarShift(NULL,timeFrame,Time[i]);
                  int x = iBarShift(NULL,timeFrame,Time[i+1]);
                     ExtMapBuffer1[i]    = iCustom(NULL,timeFrame,indicatorFileName,"",FractalBars,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,0,y);
                     ExtMapBuffer2[i]    = iCustom(NULL,timeFrame,indicatorFileName,"",FractalBars,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,1,y);
                     ArrowUpBuffer3[i]   = EMPTY_VALUE;
                     ArrowDownBuffer4[i] = EMPTY_VALUE;
                     if (x!=y)
                     {
                        ArrowUpBuffer3[i]   = iCustom(NULL,timeFrame,indicatorFileName,"",FractalBars,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,2,y);
                        ArrowDownBuffer4[i] = iCustom(NULL,timeFrame,indicatorFileName,"",FractalBars,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,3,y);
                     }
               }
               return(0);
           }

   for(i = limit; i > 0; i--)
   {
      ArrowUpBuffer3[i]  = EMPTY_VALUE;
      ArrowDownBuffer4[i]= EMPTY_VALUE;
      ExtMapBuffer1[i] = isFractalUp(i, FractalBars);
      if(ExtMapBuffer1[i] == 0)
         ExtMapBuffer1[i] = ExtMapBuffer1[i+1];
      else
         ArrowUpBuffer3[i] = ExtMapBuffer1[i];
         
      
      ExtMapBuffer2[i] = isFractalDown(i, FractalBars);
      if(ExtMapBuffer2[i] == 0)
         ExtMapBuffer2[i] = ExtMapBuffer2[i+1];
      else
         ArrowDownBuffer4[i] = ExtMapBuffer2[i];
         
   }
   manageAlerts();
   return(0);
}


double isFractalUp(int index, int bars)
{
   double max = High[index];
   for(int i = index + bars; i >= (index - bars); i--)
   {
      if(High[i] == 0.0)return(0);
      if(max <= High[i] && i != index)
      {
         if(max < High[i])
            return(0);
         if(MathAbs(i - index) > 1)
            return(0);
      }
   }
   return(max);
}


double isFractalDown(int index, int bars)
{
   double min = Low[index];
   for(int i = index + bars; i >= (index - bars); i--)
   {
      if(Low[i] == 0.0)return(0);
      if(min >= Low[i] && i != index)
      {
         if(min > Low[i])
            return(0);

         if(MathAbs(i - index) > 1)
            return(0);
      }
   }
   return(min);
}

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = FractalBars;
      else     whichBar = FractalBars+1;
      if (ArrowUpBuffer3[whichBar] != EMPTY_VALUE || ArrowDownBuffer4[whichBar] != EMPTY_VALUE)
      {
         if (ArrowUpBuffer3[whichBar]   != EMPTY_VALUE) doAlert(whichBar,"sell");
         if (ArrowDownBuffer4[whichBar] != EMPTY_VALUE) doAlert(whichBar,"buy");
      }
   }
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(timeFrameToString(Period())+" "+Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," FractalLevels fractal changed to ",doWhat);
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol()," FractalLevels"),message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}